==> api_server_controller-0.2.3a7-py3-none-any.whl: asc/tools/marketplace/data.py <==
from typing import List
from typing import Tuple
from typing import Union
from typing import NewType
from dataclasses import dataclass

from asc.core.ABC.Serializer import DataSerializer
from asc.tools.marketplace.mappings import ProcessType

TaxRow = NewType('TaxRow', Union[
    Tuple[ProcessType, float],
    Tuple[ProcessType, List[float]]
])

TaxRows = NewType('TaxRows', Union[
    None,  # Sem taxa.
    TaxRow,  # Taxa única a ser cobrada.
    List[TaxRow]  # Taxas em sequencia a ser cobrada.
])


@dataclass
class Installment(DataSerializer):
    """ Objeto de parcela. """

    id:int
    tax:TaxRows  # Taxas atreladas.
    count:int
    price:int  # Preço final da parcela.
    total:Union[int,float]  # Preço total da cobrança.
    total_tax:int  # Preço total final da cobrança (com taxas).
    interest:bool = False  # Houve taxas.
    subtract:bool = False  # Subtrair taxas.
    is_serie:bool = False  # Taxas calculadas em série.

    def __init__(self, count:int, tax:TaxRows, total, subtract=False, in_serie=False):
        self.id = count
        self.tax = tax
        self.count = count
        self.total = total
        self.total_tax = total
        self.subtract = subtract
        self.in_serie = in_serie

        self.calc_taxes()  # Calcula taxas para obter valor da parcela.

        self.price = round(self.total_tax / count,2)

    def __repr__(self):
        txt_total_tax = 'recebido' if self.subtract else 'a_ser_pago'
        return f'<' \
               f'valor_a_receber={float(self.total)} ' \
               f'qnt_vezes={self.id} ' \
               f'vlr_taxas={abs(round(self.total - self.total_tax, 2))} '\
               f'vlr_parcela={self.price} ' \
               f'vlr_{txt_total_tax}={self.total_tax} ' \
               f'>'

    def get_total(self):
        return self.total_tax if self.in_serie else self.total

    def get_total_without_tax(self, process_type:int, tax:float):
        """ Remove taxas do total com valores específicos.
        :param process_type: Processar taxa como Fixo ou percentual.
        :param tax: Valor taxado.
        :return: total sem taxa. """

        total = self.get_total()

        if process_type == ProcessType.fix.value:
            self.total_tax -= float(tax)
        if process_type == ProcessType.percentage.value:
            self.total_tax -= round(total * (float(tax) / 100), 2)

        return self.total_tax

    def get_total_with_tax(self, process_type:int, tax:float):
        """ Soma taxa ao total com valores específicos.
        :param process_type: Processar taxa como Fixo ou percentual.
        :param tax: Valor a ser taxado.
        :return: total com taxa. """

        total = self.get_total()

        if process_type == ProcessType.fix.value:
            self.total_tax += float(tax)
        if process_type == ProcessType.percentage.value:
            self.total_tax += round(total * (float(tax) / 100), 2)

        return self.total_tax

    def calc_tax(self, process_type:ProcessType, tax:float):
        """ Calcula taxa com valores específicos.
        :param process_type: Processar taxa como Fixo ou percentual.
        :param tax: Valor a ser taxado.
        :return: total com taxa. """

        if self.subtract:
            self.get_total_without_tax(process_type, tax)
        else:
            self.get_total_with_tax(process_type, tax)

        self.interest = True
        self.total_tax = round(self.total_tax, 2)
        return self.total_tax

    def calc_taxes(self) -> True:
        """ Calcula taxas em sequencia.
        :return: True quando houver taxas."""

        if not self.tax:
            return self.total_tax

        if isinstance(self.tax, list):
            for tax_row in self.tax:
                if isinstance(tax_row[1], list):
                    for t in tax_row[1]:
                        self.calc_tax(tax_row[0], t)
                else:
                    self.calc_tax(tax_row[0], tax_row[1])
        else:
            self.calc_tax(self.tax[0], self.tax[1])

        return True

==> api_server_controller-0.2.4a6-py3-none-any.whl: asc/tools/marketplace/data.py <==
from typing import List
from typing import Tuple
from typing import Union
from typing import NewType
from dataclasses import dataclass

from asc.core import env
from asc.core.ABC.Serializer import DataSerializer
from asc.tools.marketplace.mappings import ProcessType

TaxRow = NewType('TaxRow', Union[
    Tuple[ProcessType, float],
    Tuple[ProcessType, List[float]]
])

TaxRows = NewType('TaxRows', Union[
    None,  # Sem taxa.
    TaxRow,  # Taxa única a ser cobrada.
    List[TaxRow]  # Taxas em sequencia a ser cobrada.
])


@dataclass
class Installment(DataSerializer):
    """ Calcula e retorna objeto de parcelas. """
    id:int
    tax:TaxRows  # Taxas atreladas.
    count:int
    price:int  # Preço final da parcela.
    total:int  # Preço total da cobrança.
    total_tax:int  # Preço total final da cobrança (com taxas).
    interest:bool = False  # Houve taxas.
    subtract:bool = False  # Subtrair taxas.
    is_serie:bool = False  # Taxas calculadas em série.

    def __init__(self, count:int, tax:TaxRows, total, subtract=False, in_serie=False):
        self.id = count
        self.tax = tax
        self.count = count
        self.total = total
        self.total_tax = total
        self.total_without_tax = total
        self.subtract = subtract
        self.in_serie = in_serie

        # Calcula taxas para obter valor da parcela.
        self.calc_taxes()

        # Calcula valor da parcela.
        self.calc_price()

    def __repr__(self):
        txt_total_tax = 'recebido' if self.subtract else 'a_ser_pago'

        if self.is_free_tax():
            vlr_taxas = 0

        return f'<' \
               f'vlr_inicial={float(self.total_without_tax)} ' \
               f'qnt_vezes={self.id} ' \
               f'vlr_taxas={self.vlr_taxas} '\
               f'vlr_parcela={self.price} ' \
               f'vlr_{txt_total_tax}={self.total_tax} ' \
               f'>'

    @property
    def vlr_taxas(self):
        return round(self.count * self.price, 2)

    def is_free_tax(self) -> bool:
        """
        Se o valor total é livre de taxas.
        :return:
        """
        return self.total == self.total_tax

    def get_total(self) -> int:
        """
        Obtém total do valor.
        aplica taxas em série (somando do total inicial), ou
        aplica taxas em paralelo (sem somar o total inicial).
        :return:
        """
        return self.total_tax if self.in_serie else self.total

    def get_total_without_tax(self, process_type:int, tax:float):
        """ Remove taxas do total com valores específicos.
        :param process_type: Processar taxa como Fixo ou percentual.
        :param tax: Valor taxado.
        :return: total sem taxa. """
        total = self.get_total()

        if process_type == ProcessType.fix.value:
            self.total_tax -= float(tax)
        if process_type == ProcessType.percentage.value:
            self.total_tax -= round(total * (float(tax) / 100), 2)

        return self.total_tax

    def get_total_with_tax(self, process_type:int, tax:float):
        """ Soma taxa ao total com valores específicos.
        :param process_type: Processar taxa como Fixo ou percentual.
        :param tax: Valor a ser taxado.
        :return: total com taxa. """
        total = self.get_total()

        if process_type == ProcessType.fix.value:
            self.total_tax += float(tax)
        if process_type == ProcessType.percentage.value:
            self.total_tax += round(total * (float(tax) / 100), 2)

        return self.total_tax

    def calc_price(self):
        """ Calcula valor de parcelas.
        """

        if self.count <= int(env.get('marketplace__installments_free',0)):
            # Remove taxa quando gratuito.
            self.total_tax = self.total
            self.interest = False
        else:
            self.total = self.total_tax

        self.price = float('{:.2f}'.format(self.total_tax / self.count))

        return self.price

    def calc_tax(self, process_type:ProcessType, tax:float):
        """ Calcula taxa com valores específicos.

        :param process_type: Processar taxa como Fixo ou percentual.
        :param tax: Valor a ser taxado.
        :return: total com taxa. """

        if self.subtract:
            self.get_total_without_tax(process_type, tax)
        else:
            self.get_total_with_tax(process_type, tax)

        self.interest = True
        self.total_tax = round(self.total_tax, 2)
        return self.total_tax

    def calc_taxes(self) -> True:
        """ Calcula e aplica taxas em sequencia.
        :return: True quando houver taxas."""

        if not self.tax:
            return self.total_tax

        if isinstance(self.tax, list):
            for tax_row in self.tax:
                if isinstance(tax_row[1], list):
                    for t in tax_row[1]:
                        self.calc_tax(tax_row[0], t)
                else:
                    self.calc_tax(tax_row[0], tax_row[1])
        else:
            self.calc_tax(self.tax[0], self.tax[1])

        return True
